// BcasCardReader.h: B-CASJ[h[_NX
//
/////////////////////////////////////////////////////////////////////////////

#pragma once

#include <Vector>
#include <String>
#include <WinScard.h>


/////////////////////////////////////////////////////////////////////////////
// ėpX}[gJ[h[_p B-CASJ[hC^tF[XNX
/////////////////////////////////////////////////////////////////////////////

class CBcasCardReader :	public CBonObject,
						public IHalDevice,
						public IHalBcasCard
{
public:
// CBonObject
	DECLARE_IBONOBJECT(CBcasCardReader)

// IHalDevice
	virtual const BONGUID GetDeviceType(void) const;
	virtual const CBonString GetDeviceName() const;
	virtual const DWORD GetTotalDeviceNum(void) const;
	virtual const DWORD GetActiveDeviceNum(void) const;

// IHalBcasCard
	virtual const bool OpenCard(void);
	virtual void CloseCard(void);
	virtual const bool IsInteractive(void);
	virtual const BYTE * GetBcasCardID(void);
	virtual const BYTE * GetInitialCbc(void);
	virtual const BYTE * GetSystemKey(void);
	virtual const BYTE * GetKsFromEcm(const BYTE *pEcmData, const DWORD dwEcmSize);
	virtual const DWORD GetEcmError(void);
	virtual void ResetCard(void);
	virtual const bool SendEmmSection(const BYTE *pEmmData, const DWORD dwEmmSize);
	virtual const WORD GetPowerCtrlInfo(POWERCTRLINFO *pPowerCtrlInfo);

// CBcasCardReader
	CBcasCardReader(IBonObject *pOwner);
	virtual ~CBcasCardReader(void);
	
protected:
	const bool EnumBcasCardReader(void) const;

private:
	const bool TransmitCommand(const SCARDHANDLE hBcasCard, const BYTE *pSendData, const DWORD dwSendSize, BYTE *pRecvData, const DWORD dwMaxRecv, DWORD *pdwRecvSize = NULL) const;
	const bool InitialSetting(const SCARDHANDLE hBcasCard) const;

	struct TAG_BCASCARDINFO
	{
		BYTE BcasCardID[6];		// Card ID
		BYTE SystemKey[32];		// Descrambling system key
		BYTE InitialCbc[8];		// Descrambler CBC initial value
	};

	struct TAG_ECMSTATUS
	{
		BYTE KsData[16];		// Ks Odd + Even	
	};

	SCARDCONTEXT m_ScardContext;
	SCARDHANDLE m_hBcasCard;

	mutable std::vector<CBonString> m_CardReaderArray;
	
	mutable TAG_BCASCARDINFO m_BcasCardInfo;
	mutable TAG_ECMSTATUS m_EcmStatus;
	mutable DWORD m_dwEcmError;
};
