
#include "stdafx.h"

#include "ManagedLock.h"
#include "DataQueue.h"
#include "RawTsData.h"
#include "Tuner.h"

namespace Bon
{
	DataQueue::DataQueue(Tuner^ parent, __int64 maxSize)
		: parent_(parent), maxSize_(maxSize), totalSize_(0),
		queue_(gcnew System::Collections::Generic::Queue<ITsData^>()),
		queued_(gcnew System::Threading::ManualResetEvent(false))
	{
	}

	DataQueue::~DataQueue()
	{
		if (parent_ != nullptr)
		{
			parent_->RemoveQueue(this);
			parent_ = nullptr;
		}

		Close();
	}

	void DataQueue::Close()
	{
		Internal::ManagedLock lock(queue_);

		queue_->Clear();
		queued_->Set();
		totalSize_ = 0;
	}

	void DataQueue::Clear()
	{
		Internal::ManagedLock lock(queue_);

		queue_->Clear();
		if (parent_ != nullptr)
		{
			queued_->Reset();
		}
		totalSize_ = 0;
	}

	System::Threading::WaitHandle^ DataQueue::CanDequeue::get()
	{
		return queued_;
	}

	System::IntPtr DataQueue::CanDequeueHandle::get()
	{
		return queued_->Handle;
	}

	int DataQueue::Count::get()
	{
		Internal::ManagedLock lock(queue_);

		return queue_->Count;
	}

	__int64 DataQueue::MaxSize::get()
	{
		return maxSize_;
	}

	__int64 DataQueue::TotalSize::get()
	{
		Internal::ManagedLock lock(queue_);

		return totalSize_;
	}

	void DataQueue::Enqueue(ITsData^ data)
	{
		Internal::ManagedLock lock(queue_);

		int size = data->Size;
		if ((totalSize_ + size) > maxSize_)
		{
			while (queue_->Count > 0)
			{
				totalSize_ -= queue_->Dequeue()->Size;
				if ((totalSize_ + size) <= maxSize_)
				{
					break;
				}
			}
		}

		queue_->Enqueue(data);
		if (queue_->Count == 1)
		{
			queued_->Set();
		}
		totalSize_ += size;
	}

	ITsData^ DataQueue::Dequeue()
	{
		queued_->WaitOne();

		Internal::ManagedLock lock(queue_);

		if (queue_->Count == 0)
		{
			throw gcnew System::ObjectDisposedException("DataQueue disposed");
		}

		ITsData^ data = queue_->Dequeue();
		if (queue_->Count == 0)
		{
			queued_->Reset();
		}
		totalSize_ -= data->Size;

		return data;
	}

	IRawTsData^ DataQueue::RawDequeue()
	{
		ITsData^ data = Dequeue();
		return gcnew RawTsData(data);
	}
}
