
#include "stdafx.h"

#include "ManagedLock.h"
#include "ManagedBuffer.h"
#include "DataQueueManager.h"
#include "TsData.h"
#include "DataQueue.h"

namespace Bon
{
	namespace Internal
	{
		DataQueueManager::DataQueueManager(IBonObject *pOwner, int bufferSize)
			: CMediaDecoder(pOwner),
			buffer_(bufferSize), position_(0),
			queueList_(gcnew System::Collections::Generic::List<DataQueue^>())
		{
		}

		DataQueueManager::~DataQueueManager()
		{
			ManagedLock lock(queueList_);

			for (int i = 0; i < queueList_->Count; i++)
			{
				queueList_->default::get(i)->Close();
			}

			queueList_->Clear();
		}

		const bool DataQueueManager::OnPlay()
		{
			Clear();
			return CMediaDecoder::OnPlay();
		}

		const bool DataQueueManager::OnStop()
		{
			Clear();
			return CMediaDecoder::OnStop();
		}

		const bool DataQueueManager::OnReset()
		{
			Clear();
			return CMediaDecoder::OnReset();
		}

		const bool DataQueueManager::OnInputMedia(const IMediaData *pMediaData, const DWORD dwInputIndex)
		{
			Internal::ManagedLock lock(queueList_);

			DWORD dataSize = pMediaData->GetSize();
			if ((position_ + dataSize) > buffer_.GetSize())
			{
				if (position_ > 0)
				{
					ITsData^ data = gcnew TsData(buffer_.GetManagedData(), position_);

					for (int i = 0; i < queueList_->Count; i++)
					{
						queueList_->default::get(i)->Enqueue(data);
					}

					buffer_.Renew();
					position_ = 0;
				}

				if (dataSize > buffer_.GetSize())
				{
					ManagedBuffer temp(dataSize);

					memcpy(temp.GetData(), pMediaData->GetData(), dataSize);

					ITsData^ data = gcnew TsData(temp.GetManagedData(), dataSize);

					for (int i = 0; i < queueList_->Count; i++)
					{
						queueList_->default::get(i)->Enqueue(data);
					}

					return true;
				}
			}

			memcpy(buffer_.GetData() + position_, pMediaData->GetData(), dataSize);
			position_ += dataSize;

			return true;
		}

		void DataQueueManager::Clear()
		{
			Internal::ManagedLock lock(queueList_);

			for (int i = 0; i < queueList_->Count; i++)
			{
				queueList_->default::get(i)->Clear();
			}

			position_ = 0;
		}

		System::Collections::Generic::KeyValuePair<IDataQueue^, int>
			DataQueueManager::CreateQueue(Tuner^ parent, __int64 maxSize)
		{
			DataQueue^ queue = gcnew DataQueue(parent, maxSize);

			Internal::ManagedLock lock(queueList_);

			queueList_->Add(queue);
			return System::Collections::Generic::KeyValuePair<IDataQueue^, int>(
				queue, queueList_->Count);
		}

		int DataQueueManager::RemoveQueue(DataQueue^ queue)
		{
			Internal::ManagedLock lock(queueList_);

			queueList_->Remove(queue);
			return queueList_->Count;
		}
	}
}
