
#include "stdafx.h"

#include "EpgData.h"

namespace Bon
{
	EpgData::EpgData(const IEpgItem *pEpgItem)
		: pEpgItem_(dynamic_cast<IEpgItem *>(BON_CREATE_INSTANCE(TEXT("CEpgItem"), 0)))
	{
		pEpgItem_->CopyItem(pEpgItem);
	}

	EpgData::~EpgData()
	{
		BON_SAFE_RELEASE(pEpgItem_);
	}

	EpgDataStates EpgData::EpgDataStatus::get()
	{
		return static_cast<EpgDataStates>(pEpgItem_->GetStoreState());
	}

	int EpgData::EventID::get()
	{
		return pEpgItem_->GetEventID();
	}

	System::DateTime EpgData::StartTime::get()
	{
		const CTsTime &time = pEpgItem_->GetStartTime();
		return System::DateTime(time.wYear, time.wMonth, time.wDay,
			time.wHour, time.wMinute, time.wSecond, time.wMilliseconds,
			System::DateTimeKind::Local);
	}

	System::TimeSpan EpgData::Duration::get()
	{
		return System::TimeSpan(0, 0, pEpgItem_->GetDuration());
	}

	bool EpgData::IsScrambled::get()
	{
		return pEpgItem_->IsScrambled();
	}

	System::String^ EpgData::EventTitle::get()
	{
		const CBonString eventTitle = pEpgItem_->GetEventTitle();
		return gcnew System::String(eventTitle);
	}

	System::String^ EpgData::EventInfo::get()
	{
		const CBonString eventInfo = pEpgItem_->GetEventInfo();
		return gcnew System::String(eventInfo);
	}

	System::String^ EpgData::EventDetail::get()
	{
		const CBonString eventDetail = pEpgItem_->GetEventDetail();
		return gcnew System::String(eventDetail);
	}

	int EpgData::VideoResolution::get()
	{
		return pEpgItem_->GetVideoResolution();
	}

	bool EpgData::IsVideoProgressive::get()
	{
		return pEpgItem_->IsVideoProgressive();
	}

	bool EpgData::IsVideoAspectWide::get()
	{
		return pEpgItem_->IsVideoAspectWide();
	}

	bool EpgData::IsVideoPanAndScan::get()
	{
		return pEpgItem_->IsVideoPanAndScan();
	}

	AudioModes EpgData::AudioMode::get()
	{
		return static_cast<AudioModes>(pEpgItem_->GetAudioMode());
	}

	int EpgData::Genre::get()
	{
		return pEpgItem_->GetGenreNum();
	}

	int EpgData::GetGenreLevel1(int index)
	{
		return pEpgItem_->GetGenreLevel1(index);
	}

	int EpgData::GetGenreLevel2(int index)
	{
		return pEpgItem_->GetGenreLevel2(index);
	}
}
