
#include "stdafx.h"

#include "ManagedLock.h"
#include "CreateBonInstance.h"
#include "Tuner.h"
#include "TunerFactory.h"
#include "TuningSpaceInformation.h"
#include "TunerDefinition.h"

namespace Bon
{
	void TunerFactory::UpdateTunerDefinitions()
	{
		IBonClassEnumerator* pBonClassEnumerator =
			Internal::CreateBonInstance<IBonClassEnumerator>(L"CBonClassEnumerator");

		try
		{
			System::Collections::Generic::List<ITunerDefinition^>^ tunerDefinitions =
				gcnew System::Collections::Generic::List<ITunerDefinition^>();

			if (pBonClassEnumerator->EnumBonClass(L"IHalTsTuner"))
			{
				DWORD index = 0;
				while (true)
				{
					const CBonString className = pBonClassEnumerator->GetBonClassName(index++);
					if (className.GetLength() == 0)
					{
						break;
					}

					IHalDevice *pHalDevice = BON_SAFE_CREATE<IHalDevice *>(className);
					if (pHalDevice != 0)
					{
						const CBonString deviceName = pHalDevice->GetDeviceName();

						if (pHalDevice->GetDeviceType() == BON_NAME_TO_GUID(TEXT("IHalTsTuner")))
						{
							IHalTsTuner *pHalTsTuner = dynamic_cast<IHalTsTuner *>(pHalDevice);

							System::Collections::Generic::List<ITuningSpaceInformation^>^ tuningSpaces =
								gcnew System::Collections::Generic::List<ITuningSpaceInformation^>();
							
							DWORD tuningSpaceIndex = 0;
							for (;;)
							{
								const CBonString tuningSpaceName = pHalTsTuner->EnumTuningSpace(tuningSpaceIndex);
								if (tuningSpaceName.IsEmpty())
								{
									break;
								}

								System::Collections::Generic::List<System::String^>^ channels =
									gcnew System::Collections::Generic::List<System::String^>();

								DWORD channelIndex = 0;
								for (;;)
								{
									const CBonString channelName = pHalTsTuner->EnumChannelName(tuningSpaceIndex, channelIndex);
									if (channelName.IsEmpty())
									{
										break;
									}
									channels->Add(gcnew System::String(channelName));
									channelIndex++;
								}

								tuningSpaces->Add(gcnew TuningSpaceInformation(
									gcnew System::String(tuningSpaceName), channels->ToArray()));
								tuningSpaceIndex++;
							}

							tunerDefinitions->Add(gcnew TunerDefinition(
								gcnew System::String(className), gcnew System::String(deviceName), tuningSpaces->ToArray()));
						}
						BON_SAFE_RELEASE(pHalDevice);
					}
				}
			}

			tunerDefinitions_ = tunerDefinitions->ToArray();
		}
		finally
		{
			BON_SAFE_RELEASE(pBonClassEnumerator);
		}
	}

	TunerFactory::TunerFactory()
		: tunerDefinitions_(nullptr)
	{
	}

	int TunerFactory::Count::get()
	{
		Internal::ManagedLock lock(this);

		if (tunerDefinitions_ == nullptr)
		{
			UpdateTunerDefinitions();
		}

		return tunerDefinitions_->Length;
	}

	ITunerDefinition^ TunerFactory::default::get(int index)
	{
		Internal::ManagedLock lock(this);

		if (tunerDefinitions_ == nullptr)
		{
			UpdateTunerDefinitions();
		}

		return tunerDefinitions_[index];
	}

	void TunerFactory::Update()
	{
		Internal::ManagedLock lock(this);

		UpdateTunerDefinitions();
	}

	System::Collections::IEnumerator^ TunerFactory::GetEnumerator()
	{
		Internal::ManagedLock lock(this);

		if (tunerDefinitions_ == nullptr)
		{
			UpdateTunerDefinitions();
		}

		return tunerDefinitions_->GetEnumerator();
	}
}
