using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.IO;
using System.Diagnostics;
using System.Net.NetworkInformation;

namespace Bon
{
	public partial class MainForm : Form
	{
		private sealed class AcceptInformatiton
		{
			private readonly TreeNode parent_;
			private readonly TreeNode node_;
			private readonly HttpListenerContext context_;

			public AcceptInformatiton(TreeNode parent, TreeNode node, HttpListenerContext context)
			{
				parent_ = parent;
				node_ = node;
				context_ = context;
			}

			public TreeNode Parent
			{
				get
				{
					return parent_;
				}
			}

			public TreeNode Node
			{
				get
				{
					return node_;
				}
			}

			public ITuner Tuner
			{
				get
				{
					return (ITuner)parent_.Tag;
				}
			}

			public HttpListenerContext Context
			{
				get
				{
					return context_;
				}
			}
		}

		private readonly HttpListener listener_ = new HttpListener();
		private readonly Dictionary<string, TreeNode> tuners_ = new Dictionary<string, TreeNode>();

		public MainForm()
		{
			InitializeComponent();

			TunerFactory factory = new TunerFactory();
			int i = 12345;
			foreach (ITunerDefinition tunerDefinition in factory)
			{
				try
				{
					ITuner tuner = tunerDefinition.CreateTuner();
					string resourceName = string.Format("/{0}/", tunerDefinition.TunerName);
					string prefix = string.Format("http://+:80/{0}/", tunerDefinition.TunerName);

					TreeNode node = new TreeNode(tunerDefinition.TunerName);
					node.Tag = tuner;
					treeView1.Nodes.Add(node);
					tuners_.Add(resourceName, node);
					listener_.Prefixes.Add(prefix);

					Trace.WriteLine("Listen: " + resourceName);

					//List<TreeNode> udps = new List<TreeNode>();
					//IPEndPoint endPoint = new IPEndPoint(IPAddress.Parse("239.255.1.1"), i);

					//DateTime now = DateTime.Now;
					//TreeNode udpNode = new TreeNode("UDP broadcast: " + endPoint.ToString());
					//udpNode.Tag = endPoint;
					//udpNode.Nodes.Add(string.Format("{0:yyyy/MM/dd HH:mm:ss}", now));
					//udpNode.Nodes.Add("0 MB sent");
					//node.Nodes.Add(udpNode);
					//udps.Add(udpNode);

					//Thread thread = new Thread(new ParameterizedThreadStart(UdpDistributeEntry));
					//thread.Start(new KeyValuePair<ITuner, IEnumerable<TreeNode>>(tuner, udps));
				}
				catch (BonException)
				{
				}
				i++;
			}

			treeView1.ExpandAll();

			listener_.Start();
			listener_.BeginGetContext(new AsyncCallback(HttpAcceptEntry), null);
		}

		private void UdpDistributeEntry(object argument)
		{
			KeyValuePair<ITuner, IEnumerable<TreeNode>> info = (KeyValuePair<ITuner, IEnumerable<TreeNode>>)argument;

			try
			{
				long sent = 0;
				DateTime latest = DateTime.Now;

				using (Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp))
				{
					//socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.Broadcast, true);
					//socket.SetSocketOption(SocketOptionLevel.IP, SocketOptionName.DontRoute, true);
					//socket.SetSocketOption(SocketOptionLevel.IP, SocketOptionName.IpTimeToLive, 1);

					using (IDataQueue queue = info.Key.CreateQueue(100 * 1024 * 1024))
					{
						foreach (TreeNode node in info.Value)
						{
							node.Nodes[1].Tag = queue;
						}

						while (true)
						{
							ITsData data = queue.Dequeue();

							for (int i = 0; i < data.Size; i += 32768)
							{
								int size = 32768;
								if ((i + size) > data.Size)
								{
									size = data.Size - i;
								}

								foreach (TreeNode node in info.Value)
								{
									IPEndPoint endPoint = (IPEndPoint)node.Tag;
									socket.SendTo(data.Data, i, size, SocketFlags.None, endPoint);
								}
							}

							sent += data.Size;

							DateTime now = DateTime.Now;
							TimeSpan span = now - latest;
							if (span.TotalSeconds > 2.0)
							{
								latest = now;
								this.BeginInvoke(new ParameterizedThreadStart(UdpUpdateContext),
									new KeyValuePair<IEnumerable<TreeNode>, long>(info.Value, sent));
							}
						}
					}
				}
			}
			catch (Exception e)
			{
				Trace.WriteLine(e.ToString());
			}
		}

		private void HttpAcceptEntry(IAsyncResult ar)
		{
			try
			{
				HttpListenerContext context = listener_.EndGetContext(ar);
				listener_.BeginGetContext(new AsyncCallback(HttpAcceptEntry), null);

				TreeNode parent = null;
				if (tuners_.TryGetValue(context.Request.Url.AbsolutePath, out parent) == false)
				{
					Trace.WriteLine("Invalid url: " + context.Request.Url.ToString());

					context.Response.StatusCode = (int)HttpStatusCode.NotFound;
					context.Response.Close();
					return;
				}

				DateTime now = DateTime.Now;
				TreeNode node = new TreeNode("HTTP from: " + context.Request.RemoteEndPoint.ToString());
				node.Tag = context.Request.RemoteEndPoint;
				node.Nodes.Add(string.Format("{0:yyyy/MM/dd HH:mm:ss}", now)).Tag = context.Request.Url;
				node.Nodes.Add("0 MB sent");

				AcceptInformatiton info = new AcceptInformatiton(parent, node, context);
				this.Invoke(new ParameterizedThreadStart(HttpBeginContext), info);

				Thread thread = new Thread(new ParameterizedThreadStart(HttpDistributeEntry));
				thread.Start(info);

				Trace.WriteLine("Accept url: " + context.Request.Url.ToString());
			}
			catch
			{
			}
		}

		private void HttpDistributeEntry(object argument)
		{
			AcceptInformatiton info = (AcceptInformatiton)argument;

			info.Context.Response.ContentType = "video/mpeg";

			try
			{
				long sent = 0;
				DateTime latest = DateTime.Now;

				using (Stream stream = info.Context.Response.OutputStream)
				{
					using (IDataQueue queue = info.Tuner.CreateQueue(100 * 1024 * 1024))
					{
						info.Node.Nodes[1].Tag = queue;

						while (true)
						{
							ITsData data = queue.Dequeue();
							stream.Write(data.Data, 0, data.Size);

							sent += data.Size;

							DateTime now = DateTime.Now;
							TimeSpan span = now - latest;
							if (span.TotalSeconds > 2.0)
							{
								latest = now;
								this.BeginInvoke(new ParameterizedThreadStart(HttpUpdateContext),
									new KeyValuePair<AcceptInformatiton, long>(info, sent));
							}
						}
					}
				}
			}
			catch
			{
			}
			finally
			{
				try
				{
					info.Context.Response.Close();
				}
				catch
				{
				}
				try
				{
					if (this.IsDisposed == false)
					{
						this.Invoke(new ParameterizedThreadStart(HttpFinishContext), info);
					}
				}
				catch
				{
				}
			}
		}

		private void UdpUpdateContext(object argument)
		{
			KeyValuePair<IEnumerable<TreeNode>, long> pair = (KeyValuePair<IEnumerable<TreeNode>, long>)argument;
			long sent = (long)pair.Value;

			try
			{
				foreach (TreeNode node in pair.Key)
				{
					node.Nodes[1].Text = string.Format("{0} MB sent", (long)(sent / 1024 / 1024));
				}
			}
			catch
			{
			}
		}

		private void HttpBeginContext(object argument)
		{
			AcceptInformatiton info = (AcceptInformatiton)argument;

			try
			{
				info.Parent.Nodes.Add(info.Node);
				info.Parent.ExpandAll();
			}
			catch
			{
			}
		}

		private void HttpUpdateContext(object argument)
		{
			KeyValuePair<AcceptInformatiton, long> pair = (KeyValuePair<AcceptInformatiton, long>)argument;
			AcceptInformatiton info = pair.Key;
			long sent = (long)pair.Value;

			try
			{
				info.Node.Nodes[1].Text = string.Format("{0} MB sent", (long)(sent / 1024 / 1024));
			}
			catch
			{
			}
		}

		private void HttpFinishContext(object argument)
		{
			AcceptInformatiton info = (AcceptInformatiton)argument;

			try
			{
				info.Parent.Nodes.Remove(info.Node);
			}
			catch
			{
			}
		}

		private void MainForm_Load(object sender, EventArgs e)
		{

		}

		private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			listener_.Prefixes.Clear();
			listener_.Stop();

			foreach (KeyValuePair<string, TreeNode> entry in tuners_)
			{
				((ITuner)entry.Value.Tag).Dispose();
			}
		}

		private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			tuners_.Clear();
		}

		private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
		{
			TreeNode node = treeView1.SelectedNode;

			if (node != null)
			{
				propertyGrid1.SelectedObject = node.Tag;
			}
			else
			{
				propertyGrid1.SelectedObject = null;
			}
		}
	}
}
