// TsDataEncode.h: TS{f[^NX̃C^[tFCX
//
//////////////////////////////////////////////////////////////////////

#pragma once


namespace Bon
{

/////////////////////////////////////////////////////////////////////////////
// ARIB STD-B24 Part1񏈗NX
/////////////////////////////////////////////////////////////////////////////

class BONAPI CAribString
{
public:
	static const CBonString AribToString(const BYTE *pSrcData, const DWORD dwSrcLen);
	
private:
	enum CODE_SET
	{
		CODE_UNKNOWN,				// sȃOtBbNZbg(Ή)
		CODE_KANJI,					// Kanji
		CODE_ALPHANUMERIC,			// Alphanumeric
		CODE_HIRAGANA,				// Hiragana
		CODE_KATAKANA,				// Katakana
		CODE_MOSAIC_A,				// Mosaic A
		CODE_MOSAIC_B,				// Mosaic B
		CODE_MOSAIC_C,				// Mosaic C
		CODE_MOSAIC_D,				// Mosaic D
		CODE_PROP_ALPHANUMERIC,		// Proportional Alphanumeric
		CODE_PROP_HIRAGANA,			// Proportional Hiragana
		CODE_PROP_KATAKANA,			// Proportional Katakana
		CODE_JIS_X0201_KATAKANA,	// JIS X 0201 Katakana
		CODE_JIS_KANJI_PLANE_1,		// JIS compatible Kanji Plane 1
		CODE_JIS_KANJI_PLANE_2,		// JIS compatible Kanji Plane 2
		CODE_ADDITIONAL_SYMBOLS		// Additional symbols
	};

	enum FONT_SIZE
	{
		SIZE_SMALL,					// SSZ	Ή
		SIZE_MEDIUM,				// MSZ	p
		SIZE_NORMAL,				// NSZ	Sp
		SIZE_UNSUPPORTED			// SZX	Ή
	};

	CODE_SET m_CodeG[4];
	CODE_SET *m_pLockingGL;
	CODE_SET *m_pLockingGR;
	CODE_SET *m_pSingleGL;
	
	FONT_SIZE m_FontSize;
	
	BYTE m_byEscSeqCount;
	BYTE m_byEscSeqIndex;
	bool m_bIsEscSeqDrcs;
	
	CAribString(void);
	
	const CBonString AribToStringInternal(const BYTE *pSrcData, const DWORD dwSrcLen);
	inline const void ProcessCharCode(CBonString &dst, const WORD wCode, const CODE_SET CodeSet);

	inline const void PutKanjiChar(CBonString &dst, const WORD wCode);
	inline const void PutAlphanumericChar(CBonString &dst, const WORD wCode);
	inline const void PutHiraganaChar(CBonString &dst, const WORD wCode);
	inline const void PutKatakanaChar(CBonString &dst, const WORD wCode);
	inline const void PutJisKatakanaChar(CBonString &dst, const WORD wCode);
	inline const void PutSymbolsChar(CBonString &dst, const WORD wCode);

	inline const void ProcessCtrlC0(CBonString &dst, const BYTE *pCode, DWORD &dwSrcPos);
	inline const void ProcessCtrlC1(CBonString &dst, const BYTE *pCode, DWORD &dwSrcPos);

	inline void ProcessEscapeSeq(const BYTE byCode);
	
	inline void LockingShiftGL(const BYTE byIndexG);
	inline void LockingShiftGR(const BYTE byIndexG);
	inline void SingleShiftGL(const BYTE byIndexG);

	inline const bool DesignationGSET(const BYTE byIndexG, const BYTE byCode);
	inline const bool DesignationDRCS(const BYTE byIndexG, const BYTE byCode);
};


/////////////////////////////////////////////////////////////////////////////
// ARIB STD-B10 Part2 Annex C MJD+JTC NX
/////////////////////////////////////////////////////////////////////////////

class BONAPI CAribTime
{
public:
	static void AribToSystemTime(SYSTEMTIME *pSysTime, const BYTE *pHexData);
	static void SplitAribMjd(const WORD wAribMjd, WORD *pwYear, WORD *pwMonth, WORD *pwDay, WORD *pwDayOfWeek = NULL);
	static void SplitAribBcd(const BYTE *pAribBcd, WORD *pwHour, WORD *pwMinute, WORD *pwSecond = NULL);
	static const DWORD AribBcdToSecond(const BYTE *pAribBcd);
};


/////////////////////////////////////////////////////////////////////////////
// SYSTEMTIME bvNX
/////////////////////////////////////////////////////////////////////////////

class BONAPI CTsTime : public SYSTEMTIME
{
public:
	CTsTime();
	CTsTime(const SYSTEMTIME &SystemTime);
	CTsTime(const ULONGLONG llFileTime);
	CTsTime(const BYTE *pHexData);
	CTsTime(const WORD wYear, const WORD wMonth, const WORD wDay, const WORD wHour = 0U, const WORD wMinute = 0U, const WORD wSecond = 0U);
	
	void SetNowTime(void);
	void SetAribTime(const BYTE *pHexData);
	void SetTime(const WORD wYear, const WORD wMonth, const WORD wDay, const WORD wHour = 0U, const WORD wMinute = 0U, const WORD wSecond = 0U);

	void ClearTime(void);
	const bool IsEmpty(void) const;

	operator const ULONGLONG () const;

	const CTsTime & operator = (const SYSTEMTIME &SystemTime);
	const CTsTime & operator = (const ULONGLONG llFileTime);

	const CTsTime operator + (const DWORD dwSecond) const;
	const CTsTime operator - (const DWORD dwSecond) const;
	const CTsTime & operator += (const DWORD dwSecond);
	const CTsTime & operator -= (const DWORD dwSecond);

	const bool operator == (const CTsTime &Operand) const;
	const bool operator != (const CTsTime &Operand) const;
	const bool operator < (const CTsTime &Operand) const;
	const bool operator > (const CTsTime &Operand) const;
	const bool operator <= (const CTsTime &Operand) const;
	const bool operator >= (const CTsTime &Operand) const;
};

}
