// BonString.h: NX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "BonString.h"

#include <tchar.h>
#include <stdarg.h>
#include <atlbase.h>

namespace Bon
{

/////////////////////////////////////////////////////////////////////////////
// ϊNX
/////////////////////////////////////////////////////////////////////////////

template <typename T> class CharToString
{
private:
	T m_Buffer[2];

public:
	CharToString(const T value, const T nil)
	{
		m_Buffer[0] = value;
		m_Buffer[1] = nil;
	}

	CharToString(const CharToString &rhs)
		: m_Buffer(rhs.m_Buffer)
	{
	}

	operator const T *() const
	{
		return m_Buffer;
	}
};

static inline CharToString<char> ToString(const char value)
{
	return CharToString<char>(value, '\0');
}

static inline CharToString<wchar_t> ToString(const wchar_t value)
{
	return CharToString<wchar_t>(value, L'\0');
}


/////////////////////////////////////////////////////////////////////////////
// NXiێEQƃJEgǗj
/////////////////////////////////////////////////////////////////////////////

class CBonString::String
{
public:
#ifdef _UNICODE
	typedef std::wstring RawString;
#else
	typedef std::string RawString;
#endif

private:
	RawString m_String;
	LONG m_Count;

	String(const String &);
	void operator =(const String &);

	~String()
	{
	}

public:
	String()
		: m_Count(1)
	{
	}

	String(const char *pString)
		: m_String(ATL::CA2T(pString)), m_Count(1)
	{
	}

	String(const wchar_t *pString)
		: m_String(ATL::CW2T(pString)), m_Count(1)
	{
	}

	String(const RawString &rhs)
		: m_String(rhs), m_Count(1)
	{
	}

	String *AddRef()
	{
		InterlockedIncrement(&m_Count);
		return this;
	}

	void Release()
	{
		LONG result = InterlockedDecrement(&m_Count);
		if (result == 0)
		{
			delete this;
		}
	}

	RawString &GetString()
	{
		return m_String;
	}

	const RawString &GetString() const
	{
		return m_String;
	}
};


/////////////////////////////////////////////////////////////////////////////
// NX
/////////////////////////////////////////////////////////////////////////////

// {IɕێĂ镶̓CX^XԂŋLĂB
// ̂߁Aʂ̃Rs[ĂAقƂǃI[o[wbhȂB
// ύXKvȏꍇ͂̃\bhĂяoāALĂ镶̃Rs[
// 쐬ĕAɑ΂ĕύXsB
void CBonString::Fork()
{
	String *pNewString = new String(m_pString->GetString());
	m_pString->Release();
	m_pString = pNewString;
}

CBonString::CBonString()
	: m_pString(new String())
{
}

CBonString::CBonString(const char ch)
	: m_pString(new String(ATL::CA2T(ToString(ch))))
{
}

CBonString::CBonString(const wchar_t ch)
	: m_pString(new String(ATL::CW2T(ToString(ch))))
{
}

CBonString::CBonString(const char *pString)
	: m_pString(new String(ATL::CA2T(pString)))
{
}

CBonString::CBonString(const wchar_t *pString)
	: m_pString(new String(ATL::CW2T(pString)))
{
}

CBonString::CBonString(const CBonString &rhs)
	: m_pString(rhs.m_pString->AddRef())
{
}

CBonString::~CBonString()
{
	m_pString->Release();
}

const int CBonString::GetLength() const
{
	return static_cast<int>(m_pString->GetString().length());
}

const bool CBonString::IsEmpty() const
{
	return m_pString->GetString().length() == 0;
}

void CBonString::Clear()
{
	m_pString->Release();
	m_pString = new String();
}

TCHAR &CBonString::operator [](int index)
{
	Fork();
	return m_pString->GetString().at(index);
}

const TCHAR &CBonString::operator [](int index) const
{
	return m_pString->GetString().at(index);
}

CBonString::operator const TCHAR *() const
{
	return m_pString->GetString().c_str();
}

const TCHAR *CBonString::GetString() const
{
	return m_pString->GetString().c_str();
}

CBonString::operator const bool() const
{
	return m_pString->GetString().length() > 0;
}

const bool CBonString::operator !() const
{
	return m_pString->GetString().length() == 0;
}

CBonString &CBonString::operator =(const char ch)
{
	m_pString->Release();
	m_pString = new String(ATL::CA2T(ToString(ch)));

	return *this;
}

CBonString &CBonString::operator =(const wchar_t ch)
{
	m_pString->Release();
	m_pString = new String(ATL::CW2T(ToString(ch)));

	return *this;
}

CBonString &CBonString::operator =(const char *pString)
{
	m_pString->Release();
	m_pString = new String(pString);

	return *this;
}

CBonString &CBonString::operator =(const wchar_t *pString)
{
	m_pString->Release();
	m_pString = new String(pString);

	return *this;
}

CBonString &CBonString::operator =(const CBonString &rhs)
{
	m_pString->Release();
	m_pString = rhs.m_pString->AddRef();

	return *this;
}

CBonString &CBonString::operator +=(const char ch)
{
	Fork();
	m_pString->GetString().append(ATL::CA2T(ToString(ch)));

	return *this;
}

CBonString &CBonString::operator +=(const wchar_t ch)
{
	Fork();
	m_pString->GetString().append(ATL::CW2T(ToString(ch)));

	return *this;
}

CBonString &CBonString::operator +=(const char *pString)
{
	Fork();
	m_pString->GetString().append(ATL::CA2T(pString));

	return *this;
}

CBonString &CBonString::operator +=(const wchar_t *pString)
{
	Fork();
	m_pString->GetString().append(ATL::CW2T(pString));

	return *this;
}

CBonString &CBonString::operator +=(const CBonString &rhs)
{
	Fork();
	m_pString->GetString().append(rhs.m_pString->GetString());

	return *this;
}

CBonString operator +(const CBonString &lhs, const char ch)
{
	return CBonString(lhs) += ch;
}

CBonString operator +(const CBonString &lhs, const wchar_t ch)
{
	return CBonString(lhs) += ch;
}

CBonString operator +(const CBonString &lhs, const char *pString)
{
	return CBonString(lhs) += pString;
}

CBonString operator +(const CBonString &lhs, const wchar_t *pString)
{
	return CBonString(lhs) += pString;
}

CBonString operator +(const char ch, const CBonString &rhs)
{
	return CBonString(ch) += rhs;
}

CBonString operator +(const wchar_t ch, const CBonString &rhs)
{
	return CBonString(ch) += rhs;
}

CBonString operator +(const char *pString, const CBonString &rhs)
{
	return CBonString(pString) += rhs;
}

CBonString operator +(const wchar_t *pString, const CBonString &rhs)
{
	return CBonString(pString) += rhs;
}

CBonString operator +(const CBonString &lhs, const CBonString &rhs)
{
	return CBonString(lhs) += rhs;
}

CBonString CBonString::Format(const char *pFormat, ...)
{
	va_list va;
	va_start(va, pFormat);

	const CBonString temp = FormatV(pFormat, va);

	va_end(va);

	return temp;
}

CBonString CBonString::Format(const wchar_t *pFormat, ...)
{
	va_list va;
	va_start(va, pFormat);

	const CBonString temp = FormatV(pFormat, va);

	va_end(va);

	return temp;
}

CBonString CBonString::FormatV(const char *pFormat, va_list va)
{
	char *pBuffer = 0;
	FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_STRING,
		pFormat, 0, 0, reinterpret_cast<LPSTR>(&pBuffer), 0, &va);

	const CBonString temp(pBuffer);

	LocalFree(pBuffer);

	return temp;
}

CBonString CBonString::FormatV(const wchar_t *pFormat, va_list va)
{
	wchar_t *pBuffer = 0;
	FormatMessageW(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_STRING,
		pFormat, 0, 0, reinterpret_cast<LPWSTR>(&pBuffer), 0, &va);

	const CBonString temp(pBuffer);

	LocalFree(pBuffer);

	return temp;
}

CBonString &CBonString::AppendFormat(const char *pFormat, ...)
{
	va_list va;
	va_start(va, pFormat);

	AppendFormatV(pFormat, va);

	va_end(va);

	return *this;
}

CBonString &CBonString::AppendFormat(const wchar_t *pFormat, ...)
{
	va_list va;
	va_start(va, pFormat);

	AppendFormatV(pFormat, va);

	va_end(va);

	return *this;
}

CBonString &CBonString::AppendFormatV(const char *pFormat, va_list va)
{
	char *pBuffer = 0;
	FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_STRING,
		pFormat, 0, 0, reinterpret_cast<LPSTR>(&pBuffer), 0, &va);

	Fork();
	m_pString->GetString().append(ATL::CA2T(pBuffer));

	LocalFree(pBuffer);

	return *this;
}

CBonString &CBonString::AppendFormatV(const wchar_t *pFormat, va_list va)
{
	wchar_t *pBuffer = 0;
	FormatMessageW(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_STRING,
		pFormat, 0, 0, reinterpret_cast<LPWSTR>(&pBuffer), 0, &va);

	Fork();
	m_pString->GetString().append(ATL::CW2T(pBuffer));

	LocalFree(pBuffer);

	return *this;
}

bool operator ==(const CBonString &lhs, const char ch)
{
	return ::_tcscmp(lhs, ATL::CA2T(ToString(ch))) == 0;
}

bool operator ==(const CBonString &lhs, const wchar_t ch)
{
	return ::_tcscmp(lhs, ATL::CW2T(ToString(ch))) == 0;
}

bool operator ==(const CBonString &lhs, const char *pString)
{
	return ::_tcscmp(lhs, ATL::CA2T(pString)) == 0;
}

bool operator ==(const CBonString &lhs, const wchar_t *pString)
{
	return ::_tcscmp(lhs, ATL::CW2T(pString)) == 0;
}

bool operator ==(const char ch, const CBonString &rhs)
{
	return ::_tcscmp(ATL::CA2T(ToString(ch)), rhs) == 0;
}

bool operator ==(const wchar_t ch, const CBonString &rhs)
{
	return ::_tcscmp(ATL::CW2T(ToString(ch)), rhs) == 0;
}

bool operator ==(const char *pString, const CBonString &rhs)
{
	return ::_tcscmp(ATL::CA2T(pString), rhs) == 0;
}

bool operator ==(const wchar_t *pString, const CBonString &rhs)
{
	return ::_tcscmp(ATL::CW2T(pString), rhs) == 0;
}

bool operator ==(const CBonString &lhs, const CBonString &rhs)
{
	return ::_tcscmp(lhs, rhs) == 0;
}

bool operator !=(const CBonString &lhs, const char ch)
{
	return ::_tcscmp(lhs, ATL::CA2T(ToString(ch))) != 0;
}

bool operator !=(const CBonString &lhs, const wchar_t ch)
{
	return ::_tcscmp(lhs, ATL::CW2T(ToString(ch))) != 0;
}

bool operator !=(const CBonString &lhs, const char *pString)
{
	return ::_tcscmp(lhs, ATL::CA2T(pString)) != 0;
}

bool operator !=(const CBonString &lhs, const wchar_t *pString)
{
	return ::_tcscmp(lhs, ATL::CW2T(pString)) != 0;
}

bool operator !=(const char ch, const CBonString &rhs)
{
	return ::_tcscmp(ATL::CA2T(ToString(ch)), rhs) != 0;
}

bool operator !=(const wchar_t ch, const CBonString &rhs)
{
	return ::_tcscmp(ATL::CW2T(ToString(ch)), rhs) != 0;
}

bool operator !=(const char *pString, const CBonString &rhs)
{
	return ::_tcscmp(ATL::CA2T(pString), rhs) != 0;
}

bool operator !=(const wchar_t *pString, const CBonString &rhs)
{
	return ::_tcscmp(ATL::CW2T(pString), rhs) != 0;
}

bool operator !=(const CBonString &lhs, const CBonString &rhs)
{
	return ::_tcscmp(lhs, rhs) != 0;
}

const CBonString &CBonString::GetEmpty()
{
	static const CBonString empty;

	return empty;
}

}
