// SmartThread.cpp: X}[gXbhNX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "SmartThread.h"

#include <tchar.h>
#include <process.h>
#include <stdarg.h>
#include <comdef.h>
#include <excpt.h>
#include <eh.h>

#include <atlbase.h>
#include <exception>


namespace Bon
{

/////////////////////////////////////////////////////////////////////////////
// tOVOiNX
/////////////////////////////////////////////////////////////////////////////

CFlagSignal::CFlagSignal(DWORD dwResolution)
	: m_dwResolution(max(dwResolution, 1)), m_bFlag(false)
{
}

void CFlagSignal::Release()
{
	delete this;
}

const IKernelObject *CFlagSignal::GetKernelObject() const
{
	return 0;
}

const bool CFlagSignal::Set()
{
	m_bFlag = true;
	return true;
}

const bool CFlagSignal::Reset()
{
	m_bFlag = false;
	return true;
}

CFlagSignal::operator bool() const
{
	return m_bFlag;
}

bool CFlagSignal::operator !() const
{
	return m_bFlag == false;
}

const IWaitableObject::WaitResult CFlagSignal::Wait(const DWORD dwTimeout, const bool pumpMessage) const
{
	static const CSmartEvent dummyEvent(true);

	switch (dwTimeout)
	{
	case 0:
		if (m_bFlag == true)
		{
			return Signal0;
		}
		break;

	case INFINITE:
		while (m_bFlag == false)
		{
			dummyEvent.Wait(m_dwResolution, pumpMessage);
		}
		return Signal0;

	default:
		// ̂悤ȂȂ̂ŁAmł͂Ȃ
		for (DWORD time = 0; time < dwTimeout; time += m_dwResolution)
		{
			if (m_bFlag == true)
			{
				return Signal0;
			}
			dummyEvent.Wait(m_dwResolution, pumpMessage);
		}
		break;
	}

	return Timeout;
}


/////////////////////////////////////////////////////////////////////////////
// CxgVOiNX
/////////////////////////////////////////////////////////////////////////////

CEventSignal::CEventSignal()
	: m_Event(true)
{
}

void CEventSignal::Release()
{
	delete this;
}

const IKernelObject *CEventSignal::GetKernelObject() const
{
	return &m_Event;
}

const bool CEventSignal::Set()
{
	return m_Event.SetEvent();
}

const bool CEventSignal::Reset()
{
	return m_Event.ResetEvent();
}

CEventSignal::operator bool() const
{
	return m_Event.Wait(0) == Signal0;
}

bool CEventSignal::operator !() const
{
	return m_Event.Wait(0) != Signal0;
}

const IWaitableObject::WaitResult CEventSignal::Wait(const DWORD dwTimeout, const bool pumpMessage) const
{
	return m_Event.Wait(dwTimeout, pumpMessage);
}


/////////////////////////////////////////////////////////////////////////////
// X}[gXbhNX
/////////////////////////////////////////////////////////////////////////////

CSmartThreadBase::CSmartThreadBase(ISignalInstance *pKillSignal)
	: m_ThreadParam(0)
	, m_dwThreadID(0UL)
	, m_pKillSignal(pKillSignal)
	, m_SyncSignal(true)
{
}

CSmartThreadBase::~CSmartThreadBase(void)
{
	EndThread();
	BON_SAFE_RELEASE(m_pKillSignal);
}

const bool CSmartThreadBase::SetPriority(int nPriority)
{
	BOOL result = ::SetThreadPriority(UnsafeGetHandle(), nPriority);
	return result == TRUE;
}

const bool CSmartThreadBase::EndThread(const bool pumpMessage)
{
	if(!IsAssigned())return false;

	// IVOiZbg
	m_pKillSignal->Set();

	// Xbh̒Ăяoꍇ͊JȂ
	if(::GetCurrentThreadId() == m_dwThreadID)return true;

	// TXyhԂ̂Ƃ̓W[
	::ResumeThread(UnsafeGetHandle());

	// I҂
	// XbhŔs񓯊I/OĂȂꍇAXbh͊SɏIłȂB
	// ̏ꍇAŉiɑ҂ƂɂȂ邽ߒӂKvB
	Wait(INFINITE, pumpMessage);

	// nh
	Close();
	m_dwThreadID = 0UL;

	return true;	
}

const bool CSmartThreadBase::SuspendThread(void)
{
	if(!IsAssigned())false;

	// XbhTXyh
	::SuspendThread(UnsafeGetHandle());

	return true;	
}

const bool CSmartThreadBase::ResumeThread(void)
{
	if(!IsAssigned())false;

	// XbhW[
	::ResumeThread(UnsafeGetHandle());

	return true;	
}

const DWORD CSmartThreadBase::GetThreadID(void) const
{
	// XbhIDԂ
	return m_dwThreadID;
}

const bool CSmartThreadBase::IsRunning(void) const
{
	// XbhsLԂ
	return (m_SyncSignal.Wait(0) == Signal0) && (Wait(0) == Timeout);
}

const bool CSmartThreadBase::StartThread(const ULONG_PTR threadParam, const bool bStart)
{
	// JĂȂꍇ͊J
	EndThread();

	// p[^ۑ
	m_ThreadParam = threadParam;
	
	// VOiNA
	m_SyncSignal.ResetEvent();
	m_pKillSignal->Reset();
	
	// XbhN
	unsigned int threadID = 0;
	HANDLE hThread = reinterpret_cast<HANDLE>(_beginthreadex(
		NULL, 0UL, CSmartThreadBase::ThreadFunc, static_cast<void *>(this), (bStart)? 0UL : CREATE_SUSPENDED, &threadID));
	if(!hThread)
	{
		// XbhNs
		m_ThreadParam = 0;
		return false;
	}

	// XbhN҂
	if(bStart)
	{
		m_SyncSignal.Wait();
	}	

	Assign(hThread);
	m_dwThreadID = threadID;

	return true;
}

class CStructuredException
{
private:
	const EXCEPTION_RECORD m_Record;
	const CONTEXT m_Context;

public:
	CStructuredException(const _EXCEPTION_POINTERS *pExceptionPointers)
		: m_Record(*pExceptionPointers->ExceptionRecord), m_Context(*pExceptionPointers->ContextRecord)
	{
	}

	const EXCEPTION_RECORD &GetException() const
	{
		return m_Record;
	}

	const CONTEXT &GetContext() const
	{
		return m_Context;
	}
};

static void StructuredExceptionTranslator(unsigned int code, struct _EXCEPTION_POINTERS *pExceptionPointers)
{
	throw CStructuredException(pExceptionPointers);
}

static void OutputExceptionString(LPCTSTR pFormat, ...)
{
	va_list va;
	va_start(va, pFormat);

	const CBonString temp = CBonString::FormatV(pFormat, va);
	::OutputDebugString(temp);
}

unsigned int WINAPI CSmartThreadBase::ThreadFunc(void *pParam)
{
	CSmartThreadBase *pThis = static_cast<CSmartThreadBase *>(pParam);

	// TXyhԂŏI
	if(*(pThis->m_pKillSignal)){
		pThis->m_SyncSignal.ResetEvent();
		return 0UL;	
		}

	// NVOiZbg
	pThis->m_SyncSignal.SetEvent();

	_set_se_translator(StructuredExceptionTranslator);

	try
	{
		// Xbh\bhs
		pThis->ThreadEntry(*(pThis->m_pKillSignal), pThis->m_ThreadParam);
	}
	catch (const CBonException &e)
	{
		const CBonString description = e.GetDescription();
		const CBonString file = e.GetSourceFile();

		OutputExceptionString(TEXT("Uncaught %1!hs! context [%2!d!] in %3!s!(%4!d!) : %5!s!\r\n"),
			typeid(e).name(), pThis->m_dwThreadID, file.GetString(), e.GetSourceLine(), description.GetString());
	}
	catch (const _com_error &e)
	{
		OutputExceptionString(TEXT("Uncaught %1!hs! context [%2!d!] in %3!s! : HRESULT=%4!08x!, Message=%5!s!, Description=%6!s!\r\n"),
			typeid(e).name(), pThis->m_dwThreadID, static_cast<const TCHAR *>(e.Source()),
			e.Error(), e.ErrorMessage(), static_cast<const TCHAR *>(e.Description()));
	}
	catch (const std::exception &e)
	{
		OutputExceptionString(TEXT("Uncaught %1!hs! context [%2!d!] : %3!hs!\r\n"),
			typeid(e).name(), pThis->m_dwThreadID, e.what());
	}
	catch (const ATL::CAtlException &e)
	{
		OutputExceptionString(TEXT("Uncaught %1!hs! context [%2!d!] : HRESULT=%3!08x!\r\n"),
			typeid(e).name(), pThis->m_dwThreadID, e.m_hr);
	}
	catch (const CStructuredException &e)
	{
		OutputExceptionString(TEXT("Uncaught structured exception context [%1!d!] : Code=%2!08x!, Address=%3!p!\r\n"),
			pThis->m_dwThreadID,
			e.GetException().ExceptionCode,
			e.GetException().ExceptionAddress);
	}
	catch (...)
	{
		OutputExceptionString(TEXT("Uncaught unknown exception context [%1!d!]\r\n"),
			pThis->m_dwThreadID);
	}

	// NVOiNA
	pThis->m_SyncSignal.ResetEvent();

	return 0UL;	
}

}
