// SyncObject.cpp: IuWFNgbp[NX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "SyncObject.h"
#include <TChar.h>
#include <assert.h>


namespace Bon
{

//////////////////////////////////////////////////////////////////////
// NeBJZNVbp[NX
//////////////////////////////////////////////////////////////////////

CSmartLock::CSmartLock()
{
	// NeBJZNV
	::InitializeCriticalSection(&m_CriticalSection);
}

CSmartLock::~CSmartLock()
{
	// NeBJZNV폜
	::DeleteCriticalSection(&m_CriticalSection);
}

void CSmartLock::Lock() const
{
	// NeBJZNV擾
	::EnterCriticalSection(&m_CriticalSection);
}

void CSmartLock::Unlock() const
{
	// NeBJZNVJ
	::LeaveCriticalSection(&m_CriticalSection);
}
	
	
//////////////////////////////////////////////////////////////////////
// ubNXR[vbNNX
//////////////////////////////////////////////////////////////////////

CBlockLock::CBlockLock(const ILockObject &lock)
	: m_pLock(&lock)
{
	// bN擾
	m_pLock->Lock();
}

CBlockLock::CBlockLock(const ILockObject *pLock)
	: m_pLock(pLock)
{
	// bN擾
	if(m_pLock) m_pLock->Lock();
}

CBlockLock::~CBlockLock()
{
	// bNJ
	if(m_pLock) m_pLock->Unlock();
}


/////////////////////////////////////////////////////////////////////////////
// J[lIuWFNgҋ@֐
/////////////////////////////////////////////////////////////////////////////

static IWaitableObject::WaitResult InternalWait(
	const DWORD dwCount, const HANDLE *pHandles, const bool waitAll,
	const DWORD dwTimeout, const bool pumpMessage)
{
	if (dwCount == 0)
	{
		static const CSmartEvent dummyEvent(true);
		return dummyEvent.Wait(dwTimeout, pumpMessage);
	}

	if (pumpMessage == false)
	{
		return static_cast<IWaitableObject::WaitResult>(
			WaitForMultipleObjects(dwCount, pHandles, waitAll ? TRUE : FALSE, dwTimeout));
	}

	if (dwTimeout == INFINITE)
	{
		while (true)
		{
			DWORD result = MsgWaitForMultipleObjects(
				dwCount, pHandles, waitAll ? TRUE : FALSE, INFINITE, QS_ALLINPUT);
			if (result != (WAIT_OBJECT_0 + dwCount))
			{
				return static_cast<IWaitableObject::WaitResult>(result);
			}

			MSG message;
			if (PeekMessage(&message, NULL, 0, 0, PM_REMOVE))
			{
				TranslateMessage(&message);
				DispatchMessage(&message);
			}
		}
	}
	else
	{
		DWORD remains = dwTimeout;
		while (true)
		{
			DWORD tick = GetTickCount();

			DWORD result = MsgWaitForMultipleObjects(
				dwCount, pHandles, waitAll ? TRUE : FALSE, remains, QS_ALLINPUT);
			if (result != (WAIT_OBJECT_0 + dwCount))
			{
				return static_cast<IWaitableObject::WaitResult>(result);
			}

			MSG message;
			if (PeekMessage(&message, NULL, 0, 0, PM_REMOVE))
			{
				TranslateMessage(&message);
				DispatchMessage(&message);
			}

			DWORD past = GetTickCount();
			DWORD differ = 0;
			if (past >= tick)
			{
				differ = past - tick;
			}
			else
			{
				differ = (0xffffffffUL - tick) + past + 1;
			}

			if (differ >= remains)
			{
				return IWaitableObject::Timeout;
			}

			remains -= differ;
		}
	}
}


/////////////////////////////////////////////////////////////////////////////
// J[lIuWFNgC^[tFCX
/////////////////////////////////////////////////////////////////////////////

CWaitAnyObjects operator ||(const IKernelObject &lhs, const IKernelObject &rhs)
{
	return CWaitAnyObjects(lhs) || rhs;
}

CWaitAnyObjects operator ||(const IKernelObject *pLhs, const IKernelObject &rhs)
{
	return CWaitAnyObjects(pLhs) || rhs;
}

CWaitAnyObjects operator ||(const IKernelObject &lhs, const IKernelObject *pRhs)
{
	return CWaitAnyObjects(lhs) || pRhs;
}

CWaitAllObjects operator &&(const IKernelObject &lhs, const IKernelObject &rhs)
{
	return CWaitAllObjects(lhs) && rhs;
}

CWaitAllObjects operator &&(const IKernelObject *pLhs, const IKernelObject &rhs)
{
	return CWaitAllObjects(pLhs) && rhs;
}

CWaitAllObjects operator &&(const IKernelObject &lhs, const IKernelObject *pRhs)
{
	return CWaitAllObjects(lhs) && pRhs;
}


/////////////////////////////////////////////////////////////////////////////
// J[lIuWFNgANX
/////////////////////////////////////////////////////////////////////////////

CWaitObjectsBase::CWaitObjectsBase(const IKernelObject &rhs)
	: m_dwCount(0)
{
	if (rhs.IsAssigned() == true)
	{
		m_Handles[0] = rhs.UnsafeGetHandle();
		m_dwCount = 1;
	}
}

CWaitObjectsBase::CWaitObjectsBase(const IKernelObject *pRhs)
	: m_dwCount(0)
{
	if (pRhs != 0)
	{
		if (pRhs->IsAssigned() == true)
		{
			m_Handles[0] = pRhs->UnsafeGetHandle();
			m_dwCount = 1;
		}
	}
}

CWaitObjectsBase::CWaitObjectsBase(const CWaitObjectsBase &rhs)
	: m_dwCount(rhs.m_dwCount)
{
	memcpy(m_Handles, rhs.m_Handles, sizeof m_Handles);
}

void CWaitObjectsBase::Copy(const CWaitObjectsBase &rhs)
{
	m_dwCount = rhs.m_dwCount;
	memcpy(m_Handles, rhs.m_Handles, sizeof m_Handles);
}

void CWaitObjectsBase::Append(const IKernelObject &rhs)
{
	if (m_dwCount >= MAXIMUM_WAIT_OBJECTS)
	{
		assert(0);
		return;
	}

	if (rhs.IsAssigned() == true)
	{
		HANDLE handle = rhs.UnsafeGetHandle();
		m_Handles[m_dwCount] = handle;
		m_dwCount++;
	}
}

void CWaitObjectsBase::Append(const IKernelObject *pRhs)
{
	if (pRhs != 0)
	{
		if (m_dwCount >= MAXIMUM_WAIT_OBJECTS)
		{
			assert(0);
			return;
		}

		if (pRhs->IsAssigned() == true)
		{
			HANDLE handle = pRhs->UnsafeGetHandle();
			m_Handles[m_dwCount] = handle;
			m_dwCount++;
		}
	}
}

const IWaitableObject::WaitResult CWaitObjectsBase::Wait(
	const bool waitAll, const DWORD dwTimeout, const bool pumpMessage) const
{
	return InternalWait(m_dwCount, m_Handles, waitAll, dwTimeout, pumpMessage);
}


/////////////////////////////////////////////////////////////////////////////
// J[lIuWFNgNX
/////////////////////////////////////////////////////////////////////////////

CKernelObject::CKernelObject(HANDLE handle, HANDLE invalid)
	: m_Handle(handle), m_Invalid(invalid)
{
}

CKernelObject::~CKernelObject()
{
	Close();
}

void CKernelObject::Assign(HANDLE handle)
{
	Close();
	m_Handle = handle;
}

const bool CKernelObject::Close()
{
	if (m_Handle == m_Invalid)
	{
		return TRUE;
	}

	BOOL result = CloseHandle(m_Handle);
	m_Handle = m_Invalid;
	return result == TRUE;
}

const bool CKernelObject::IsAssigned() const
{
	return m_Handle != m_Invalid;
}

const HANDLE CKernelObject::UnsafeGetHandle() const
{
	return m_Handle;
}

CKernelObject::operator const bool() const
{
	return m_Handle != m_Invalid;
}

const bool CKernelObject::operator !() const
{
	return m_Handle == m_Invalid;
}

const IWaitableObject::WaitResult CKernelObject::Wait(const DWORD dwTimeout, bool pumpMessage) const
{
	return InternalWait(1, &m_Handle, false, dwTimeout, pumpMessage);
}


//////////////////////////////////////////////////////////////////////
// Cxgbp[NX
//////////////////////////////////////////////////////////////////////

CSmartEvent::CSmartEvent(const bool bManualReset, const bool bInitialState)
	: CKernelObject(::CreateEvent(NULL, (bManualReset)? TRUE : FALSE, (bInitialState)? TRUE : FALSE, NULL))
{
}

const bool CSmartEvent::Close(void)
{
	return CKernelObject::Close();
}

const bool CSmartEvent::SetEvent(void)
{
	// CxgZbg
	return (::SetEvent(UnsafeGetHandle()))? true : false;
}

const bool CSmartEvent::PulseEvent(void)
{
	// CxgZbg(1̃Xbĥ݊J)
	return (::PulseEvent(UnsafeGetHandle()))? true : false;
}
const bool CSmartEvent::ResetEvent(void)
{
	// CxgZbg
	return (::ResetEvent(UnsafeGetHandle()))? true : false;
}


/////////////////////////////////////////////////////////////////////////////
// ~[ebNXbp[NX
/////////////////////////////////////////////////////////////////////////////

CSmartMutex::CSmartMutex(void)
{
}

const bool CSmartMutex::Create(LPCTSTR lpszName)
{
	// ɂ͔Ή(vZXԂ̔r̂)
	if(!lpszName)return false;
	if(!::_tcslen(lpszName))return false;

	// ɃI[vς݂̏ꍇ̓G[
	if(IsAssigned())return false;
	
	// Mutex쐬
	HANDLE hMutex = ::CreateMutex(NULL, TRUE, lpszName);
	if(!hMutex)return false;
	
	// ̃vZXŃI[vĂꍇ̓G[
	if(::GetLastError() == ERROR_ALREADY_EXISTS){
		return false;
		}

	Assign(hMutex);

	return true;
}

const bool CSmartMutex::Close(void)
{
	return CKernelObject::Close();
}

const bool CSmartMutex::IsExist(LPCTSTR lpszName)
{
	// w肵OMutexI[v
	HANDLE hMutex = ::OpenMutex(MUTEX_ALL_ACCESS, FALSE, lpszName);
	
	if(hMutex){
		// Mutex݂͑
		::CloseHandle(hMutex);
		return true;
		}
	else{
		// Mutex݂͑Ȃ
		return false;
		}	
}

}
