// TsPacketSync.cpp: TSpPbgfR[_
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "TsPacketSync.h"


/////////////////////////////////////////////////////////////////////////////
// t@C[J萔ݒ
/////////////////////////////////////////////////////////////////////////////

#define STRIDE_FILTER	10UL		// pPbgtB^
#define BITRATE_PERIOD	1000UL		// rbg[gXV(1000ms)

// NbvtCNg}N
#define CLIPED_INCREMENT(V)			{if((V) < 0xFFFFFFFFUL)(V)++;}


/////////////////////////////////////////////////////////////////////////////
// TSpPbgfR[_
/////////////////////////////////////////////////////////////////////////////

const bool CTsPacketSync::OnInputMedia(const IMediaData *pMediaData, const DWORD dwInputIndex)
{
	CBlockLock lock(m_Lock);

	// pPbg
	SyncPacket(pMediaData->GetData(), pMediaData->GetSize());

	return true;
}

void CTsPacketSync::DiscardNullPacket(const bool bEnable)
{
	// NULLpPbg̔jL/ݒ
	m_bDiscardNullPacket = (bEnable)? true : false;
}

void CTsPacketSync::ResetStatistics(void)
{
	CBlockLock lock(m_Lock);

	// pPbgJE^Zbg
	m_dwCurStrideSize = 0UL;
	m_dwPrvStrideSize = 0UL;
	m_dwFixStrideSize = 0UL;
	m_dwFixStrideCount = 0UL;

	// vf[^Zbg
	m_dwInputPacketNum = 0UL;
	m_dwOutputPacketNum = 0UL;
	m_dwSyncErrNum = 0UL;
	m_dwFormatErrNum = 0UL;
	m_dwTransportErrNum = 0UL;
	m_dwContinuityErrNum = 0UL;
	m_dwLastUpdateTime = ::GetTickCount();
	m_dwLastInputPacket = 0UL;
	m_dwLastOutputPacket = 0UL;
	m_dwCurInputPacket = 0UL;
	m_dwCurOutputPacket = 0UL;

	::ZeroMemory(m_adwInputPacketNum, sizeof(m_adwInputPacketNum));
	::ZeroMemory(m_adwOutputPacketNum, sizeof(m_adwOutputPacketNum));
	::ZeroMemory(m_adwContinuityErrNum, sizeof(m_adwContinuityErrNum));

	// pPbgAJE^
	::FillMemory(m_abyContCounter, sizeof(m_abyContCounter), 0x10UL);
}

const DWORD CTsPacketSync::GetPacketStride(void)
{
	// pPbgԂ
	return m_dwFixStrideSize;
}

const DWORD CTsPacketSync::GetInputPacketNum(const WORD wPID)
{
	// ̓pPbgԂ(G[pPbgANULLpPbg܂)
	return (wPID < 0x2000U)? m_adwInputPacketNum[wPID] : m_dwInputPacketNum;
}

const DWORD CTsPacketSync::GetOutputPacketNum(const WORD wPID)
{
	// o̓pPbgԂ(NULLpPbg)
	return (wPID < 0x2000U)? m_adwOutputPacketNum[wPID] : m_dwOutputPacketNum;
}

const DWORD CTsPacketSync::GetInputBitrate(void)
{
	// ̓rbg[gԂ(pPbgTCY~1bԂ̓̓pPbg~8bit)
	return m_dwLastInputPacket * TS_PACKET_SIZE * 8UL;
}

const DWORD CTsPacketSync::GetOutputBitrate(void)
{
	// o̓rbg[gԂ(pPbgTCY~1bԂ̓̓pPbg~8bit)
	return m_dwLastInputPacket * TS_PACKET_SIZE * 8UL;
}

const DWORD CTsPacketSync::GetSyncErrNum(void)
{
	// G[Ԃ(pPbgE郂[h)
	return m_dwSyncErrNum;
}

const DWORD CTsPacketSync::GetFormatErrNum(void)
{
	// tH[}bgG[Ԃ(pPbgwb_KiÕ[h)
	return m_dwFormatErrNum;
}

const DWORD CTsPacketSync::GetTransportErrNum(void)
{
	// gX|[gG[Ԃ(Transport Error Indicator[h)
	return m_dwTransportErrNum;
}

const DWORD CTsPacketSync::GetContinuityErrNum(const WORD wPID)
{
	// AG[Ԃ(Continuity CountersAɂȂ郂[h)
	return (wPID < 0x2000U)? m_adwContinuityErrNum[wPID] : m_dwContinuityErrNum;
}

CTsPacketSync::CTsPacketSync(IBonObject *pOwner, IMediaDataHeap *pHeap)
	: CMediaDecoder(pOwner, 1UL, 1UL, pHeap)
	, m_TsPacket(0, pHeap)
	, m_dwCurStrideSize(0UL)
	, m_dwPrvStrideSize(0UL)
	, m_dwFixStrideSize(0UL)
	, m_dwFixStrideCount(0UL)
	, m_bDiscardNullPacket(true)
	, m_dwInputPacketNum(0UL)
	, m_dwOutputPacketNum(0UL)
	, m_dwSyncErrNum(0UL)
	, m_dwFormatErrNum(0UL)
	, m_dwTransportErrNum(0UL)
	, m_dwContinuityErrNum(0UL)
	, m_dwLastUpdateTime(0UL)
	, m_dwLastInputPacket(0UL)
	, m_dwLastOutputPacket(0UL)
	, m_dwCurInputPacket(0UL)
	, m_dwCurOutputPacket(0UL)
{
	// vf[^
	ResetStatistics();
}

CTsPacketSync::~CTsPacketSync(void)
{
	// Ȃ
}

const bool CTsPacketSync::OnPlay(void)
{
	// Ԃ
	return OnReset();
}

const bool CTsPacketSync::OnReset(void)
{
	CBlockLock lock(m_Lock);

	// pPbgTCYNA
	m_TsPacket.ClearSize();

	// vf[^Zbg
	ResetStatistics();
	
	return true;
}

void CTsPacketSync::SyncPacket(const BYTE *pStream, const DWORD dwSize)
{
	DWORD dwCurPos = 0UL;
	DWORD dwCurSize;

	while(dwCurPos < dwSize){
		dwCurSize = m_TsPacket.GetSize();

		if(!dwCurSize){
			// oCg҂
			do{
				m_dwCurStrideSize++;
				
				if(pStream[dwCurPos++] == 0x47U){
					// oCg
					m_TsPacket.AddByte(0x47U);
					break;
					}
				}
			while(dwCurPos < dwSize);
			}
		else{
			// f[^҂
			if((dwSize - dwCurPos) >= (TS_PACKET_SIZE - dwCurSize)){
				// pPbgTCYf[^
				m_TsPacket.AddData(&pStream[dwCurPos], TS_PACKET_SIZE - dwCurSize);
				m_dwCurStrideSize += (TS_PACKET_SIZE - dwCurSize);
				dwCurPos += (TS_PACKET_SIZE - dwCurSize);
				
				// wb_
				if(ParseTsPacket()){
					// pPbg

					// G[
					if((m_dwFixStrideSize != m_dwCurStrideSize) && m_dwFixStrideSize){
						// pPbgُ
						CLIPED_INCREMENT(m_dwSyncErrNum);
						SendDecoderEvent(EID_SYNC_ERR, m_dwSyncErrNum);
						}

					// pPbgtB^O
					if(m_dwCurStrideSize == m_dwPrvStrideSize){
						if(++m_dwFixStrideCount >= STRIDE_FILTER){
							// pPbgm
							m_dwFixStrideSize = m_dwCurStrideSize;
							m_dwFixStrideCount = STRIDE_FILTER;
							}
						}
					else{
						m_dwPrvStrideSize = m_dwCurStrideSize;
						m_dwFixStrideCount = 0UL;
						}

					// pPbgo
					OnTsPacket();

					// rbg[gXV
					UpdateBitrate();
					}

				// pPbgTCYNA
				m_TsPacket.ClearSize();
				m_dwCurStrideSize = 0UL;			
				}	
			else{
				// c̃f[^̓pPbgTCYɖȂ
				m_TsPacket.AddData(&pStream[dwCurPos], dwSize - dwCurPos);
				m_dwCurStrideSize += (dwSize - dwCurPos);
				dwCurPos += (dwSize - dwCurPos);
				}			
			}
		}
}

const bool CTsPacketSync::ParseTsPacket(void)
{
	// pPbg͂
	switch(m_TsPacket.ParsePacket(m_abyContCounter)){
		case ITsPacket::EC_VALID :
			// pPbg
			CLIPED_INCREMENT(m_dwInputPacketNum);
			CLIPED_INCREMENT(m_adwInputPacketNum[m_TsPacket.GetPID()]);
			CLIPED_INCREMENT(m_dwCurInputPacket);
			return true;

		case ITsPacket::EC_FORMAT :
			// tH[}bgُ
			CLIPED_INCREMENT(m_dwFormatErrNum);
			SendDecoderEvent(EID_FORMAT_ERR, m_dwFormatErrNum);
			return false;
		
		case ITsPacket::EC_TRANSPORT :
			// gX|[gG[ُ
			CLIPED_INCREMENT(m_dwTransportErrNum);
			SendDecoderEvent(EID_TRANSPORT_ERR, m_dwTransportErrNum);
			return false;
	
		case ITsPacket::EC_CONTINUITY :
			// Aُ(hbv)@pPbg̔j͂Ȃ
			CLIPED_INCREMENT(m_dwInputPacketNum);
			CLIPED_INCREMENT(m_adwInputPacketNum[m_TsPacket.GetPID()]);
			CLIPED_INCREMENT(m_dwContinuityErrNum);
			CLIPED_INCREMENT(m_adwContinuityErrNum[m_TsPacket.GetPID()]);
			CLIPED_INCREMENT(m_dwCurInputPacket);
			SendDecoderEvent(EID_CONTINUITY_ERR, m_dwContinuityErrNum);
			return true;
		
		default :
			// O
			::DebugBreak();
			return false;
		}
}

void CTsPacketSync::OnTsPacket(void)
{
	// pPbgo
	const WORD wPID = m_TsPacket.GetPID();

	// NULLpPbg͔j
	if(m_bDiscardNullPacket && (wPID == 0x1FFFU))return;

	// o̓pPbgCNg
	CLIPED_INCREMENT(m_dwOutputPacketNum);
	CLIPED_INCREMENT(m_adwOutputPacketNum[wPID]);
	CLIPED_INCREMENT(m_dwCurOutputPacket);
	
	// ʃfR[_Ƀf[^o
	OutputMedia(&m_TsPacket);
}

void CTsPacketSync::UpdateBitrate(void)
{
	// rbg[gXV
	const DWORD dwCurUpdateTime = ::GetTickCount();
	
	// XV`FbN
	if((dwCurUpdateTime - m_dwLastUpdateTime) < BITRATE_PERIOD)return;

	// JE^XV
	m_dwLastInputPacket = m_dwCurInputPacket;
	m_dwLastOutputPacket = m_dwCurOutputPacket;

	// JE^Zbg
	m_dwCurInputPacket = 0UL;
	m_dwCurOutputPacket = 0UL;
	
	// XV
	m_dwLastUpdateTime = dwCurUpdateTime;
}
